// SRShellEx.cpp : Implementation of CSRShellEx
#include "stdafx.h"
#include "SymRetShellEx.h"
#include "SRShellEx.h"
#include "SymRetDlg.h"

/////////////////////////////////////////////////////////////////////////////
// CSRShellEx

//
// construction / destruction.
//

CSRShellEx::CSRShellEx()
{
	// load the bitmap from resources.
	m_hbmpBitmap = ::LoadBitmap( _Module.GetModuleInstance(), MAKEINTRESOURCE( IDB_CONTEXTMENUBMP ) );
}

CSRShellEx::~CSRShellEx()
{
	// free.
	if ( m_hbmpBitmap )
		::DeleteObject( m_hbmpBitmap );
}

//
// IShellExtInit implementation.
//

HRESULT CSRShellEx::Initialize ( 
    LPCITEMIDLIST pidlFolder,
    LPDATAOBJECT pDataObj,
    HKEY hProgID )
{
	HRESULT			hrRetVal = E_INVALIDARG;

	// ## check whether we have to display our menu item. ##

	m_vcsFiles.clear ();

	FORMATETC			etc = { CF_HDROP, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
	STGMEDIUM			stg = { TYMED_HGLOBAL };

	if ( SUCCEEDED( pDataObj->GetData( & etc, & stg ) ) )
	{
		// get drop obj.
		HDROP				hdrop = (HDROP) ::GlobalLock ( stg.hGlobal );
		if ( hdrop )
		{
			// iterate through all the selected files.
			UINT				uiNumFiles = ::DragQueryFile( hdrop, 0xFFFFFFFF, NULL, 0 );
			for( UINT i=0; i<uiNumFiles; i ++ )
			{
				// get file details.
				CHAR			szFileName[ MAX_PATH ] = "";
				if ( ::DragQueryFile( hdrop, i, szFileName, MAX_PATH ) )
				{
					// check the file extension.

					char		drive[_MAX_DRIVE];
					char		dir[_MAX_DIR];
					char		fname[_MAX_FNAME];
					char		ext[_MAX_EXT];

					::_splitpath( szFileName, drive, dir, fname, ext );

					if ( ::stricmp( ext, ".acm" ) == 0 ||
						::stricmp( ext, ".ax" ) == 0 ||
						::stricmp( ext, ".com" ) == 0 ||
						::stricmp( ext, ".cpl" ) == 0 ||
						::stricmp( ext, ".dll" ) == 0 ||
						::stricmp( ext, ".drv" ) == 0 ||
						::stricmp( ext, ".ds" ) == 0 ||
						::stricmp( ext, ".exe" ) == 0 ||
						::stricmp( ext, ".ime" ) == 0 ||
						::stricmp( ext, ".ocx" ) == 0 ||
						::stricmp( ext, ".scr" ) == 0 ||
						::stricmp( ext, ".sys" ) == 0 ||
						::stricmp( ext, ".tsp" ) == 0 )
					{
						// check whether the file can be opened.

						FILE*			pfTest = ::fopen( szFileName, "rb" );
						if ( pfTest )
							::fclose( pfTest );

						// continue only if the file was opened.

						if ( pfTest )
						{
							// add to the list.

							m_vcsFiles.push_back( szFileName );
							hrRetVal = S_OK;
						}
					}
				}
			}

			// release lock.
			::GlobalUnlock( stg.hGlobal );
		}

		// release storage.
		::ReleaseStgMedium( & stg );
	}

	// return.
	return hrRetVal;
}

//
// IContextMenu implementation.
//

#define MACRO_HELPTEXT					"Get symbols for all selected images."
#define MACRO_HELPTEXT_U				L"Get symbols for all selected images."

HRESULT CSRShellEx::GetCommandString(
    UINT_PTR idCmd,
    UINT uFlags,
    UINT *pwReserved,
    LPSTR pszName,
    UINT cchMax
   )
{
	// return the help string.
	if ( uFlags & GCS_HELPTEXT )
	{
		// check string size.
		if ( cchMax <= sizeof( MACRO_HELPTEXT ) )
			return E_INVALIDARG;

		// return text.
		switch ( idCmd )
		{
		case 0:
			if ( uFlags & GCS_UNICODE )
				::wcscpy( (LPWSTR) pszName, MACRO_HELPTEXT_U );
			else
				::strcpy( pszName, MACRO_HELPTEXT );
			break;

		default:
			return E_INVALIDARG;
		}
	}

	// return.
	return S_OK;
}

HRESULT CSRShellEx::InvokeCommand(
    LPCMINVOKECOMMANDINFO pici
   )
{
	// check out which verb was sent.
	switch( (DWORD) pici->lpVerb )
	{
	case 0:

		// show the symbol retriever dialog.
		CSymRetDlg			dlg( pici, & m_vcsFiles );
		dlg.DoModal ();

		// return success.
		return S_OK;
	}

	// return failure.
	return E_INVALIDARG;
}

HRESULT CSRShellEx::QueryContextMenu(
    HMENU hmenu,
    UINT indexMenu,
    UINT idCmdFirst,
    UINT idCmdLast,
    UINT uFlags
)
{
	UINT			uiRetVal = 0;

	// Insert our menu item.
	if ( ( uFlags & CMF_DEFAULTONLY ) == 0 && hmenu )
	{
		// compose text.
		CHAR			szText[ MAX_PATH ] = "";
		::sprintf( szText, "Retrieve Symbols for %i image(s)...", m_vcsFiles.size() );

		// insert menu item.
		::InsertMenu( hmenu, indexMenu, MF_STRING | MF_BYPOSITION, idCmdFirst, szText );
		if ( m_hbmpBitmap )
			::SetMenuItemBitmaps( hmenu, indexMenu, MF_BYPOSITION, m_hbmpBitmap, NULL );

		// set ret val.
		uiRetVal = 1;
	}

	// return.
	return MAKE_HRESULT ( SEVERITY_SUCCESS, FACILITY_NULL, uiRetVal );
}

/////////////////////////////////////////////////////////////////////////////
